﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace PhaseOptics
{
    public partial class MainWindow : Window
    {
        const int n = 1024;
        const double extent = .95;
        const double bulgepos = n / 4;
        const double bulgewidth = n / 8;
        const double bulgeheight = .15;

        public MainWindow()
        {
            InitializeComponent();
            double[] optionA = new double[n];
            double[] optionB = new double[n];
            double[] optionC = new double[n];
            double[] optionD = new double[n];
            for (int i = 0; i < n; ++i)
            {
                double convex = Math.Sin(i * Math.PI / n) * (1 - extent);
                double concave = (1 - Math.Sin(i * Math.PI / n)) * (1 - extent);
                double up = i * extent / n;
                double down = extent - i * extent / n;

                double pos = OobSlayer((i - bulgepos) * Math.PI / bulgewidth, -Math.PI, Math.PI);
                double finger = (1 + Math.Cos(pos)) * .5 * bulgeheight;
                optionA[i] = OobSlayer(concave + up + finger, 0, 1);
                optionB[i] = OobSlayer(concave + down + finger, 0, 1);
                optionC[i] = OobSlayer(convex + up + finger, 0, 1);
                optionD[i] = OobSlayer(convex + down + finger, 0, 1);

            }
            a.Source = Pattern(optionA);
            b.Source = Pattern(optionB);
            c.Source = Pattern(optionC);
            d.Source = Pattern(optionD);
        }

        private double OobSlayer(double v1, double l, double u)
        {
            if (v1 < l) return l;
            if (v1 > u) return u;
            return v1;
        }

        private ImageSource Pattern(double[] height)
        {
            WriteableBitmap bitmap = new WriteableBitmap(n, 100, 96, 96, PixelFormats.Bgr32, null);
            bitmap.Lock();

            // Yellow line
            for (int i = 0; i < n; ++i)
                OrPixel(bitmap, i, (int)((1.0 - height[i]) * 99), 0xffff000);

            // Interference
            for (int i = 0; i < n; ++i)
            {
                double phaseShift = height[i] * 16 * Math.PI;
                double intensity = (Math.Cos(phaseShift) + 1) * .5;
                for (int j = 0; j < 10; j++)
                    OrPixel(bitmap, i, j, (uint)(intensity * 255));
            }

            bitmap.AddDirtyRect(new Int32Rect(0, 0, n, 100));
            bitmap.Unlock();
            return bitmap;
        }

        private void OrPixel(WriteableBitmap bitmap, int column, int row, uint value)
        {
            var target = bitmap.BackBuffer + row * bitmap.BackBufferStride + column * 4;
            unsafe { *(uint*)target = (*(uint*)target) | value; }
        }
    }
}
